@@ -144,7 +144,7 @@ dzl_preferences_view_refilter (DzlPreferencesView *self,
                          dzl_preferences_view_refilter_cb,
                          spec);
 
-  dzl_clear_pointer (&spec, dzl_pattern_spec_unref);
+  g_clear_pointer (&spec, dzl_pattern_spec_unref);
 }
 
 static gint
@@ -221,8 +221,8 @@ dzl_preferences_view_finalize (GObject *object)
   DzlPreferencesView *self = (DzlPreferencesView *)object;
   DzlPreferencesViewPrivate *priv = dzl_preferences_view_get_instance_private (self);
 
-  dzl_clear_pointer (&priv->pages, g_sequence_free);
-  dzl_clear_pointer (&priv->widgets, g_hash_table_unref);
+  g_clear_pointer (&priv->pages, g_sequence_free);
+  g_clear_pointer (&priv->widgets, g_hash_table_unref);
   g_clear_object (&priv->actions);
 
   G_OBJECT_CLASS (dzl_preferences_view_parent_class)->finalize (object);
@@ -573,8 +573,6 @@ dzl_preferences_view_add_radio (DzlPreferences *preferences,
 
       if (variant == NULL)
         g_warning ("%s", error->message);
-      else
-        g_variant_ref_sink (variant);
     }
 
   widget = g_object_new (DZL_TYPE_PREFERENCES_SWITCH,
@@ -651,8 +649,6 @@ dzl_preferences_view_add_switch (DzlPreferences *preferences,
 
       if (variant == NULL)
         g_warning ("%s", error->message);
-      else
-        g_variant_ref_sink (variant);
     }
 
   widget = g_object_new (DZL_TYPE_PREFERENCES_SWITCH,
