@@ -43,13 +43,18 @@ typedef struct
   gpointer                   func_data;
   GDestroyNotify             func_data_destroy;
 
+  guint                      activate_on_single_click : 1;
+
   gint                       in_key_press;
+  gint                       in_move_by;
 } DzlSuggestionEntryPrivate;
 
 enum {
   PROP_0,
+  PROP_ACTIVATE_ON_SINGLE_CLICK,
   PROP_MODEL,
   PROP_TYPED_TEXT,
+  PROP_SUGGESTION,
   N_PROPS
 };
 
@@ -59,6 +64,7 @@ enum {
   MOVE_SUGGESTION,
   SHOW_SUGGESTIONS,
   SUGGESTION_ACTIVATED,
+  SUGGESTION_SELECTED,
   N_SIGNALS
 };
 
@@ -75,6 +81,34 @@ static guint signals [N_SIGNALS];
 static guint changed_signal_id;
 static GtkEditableInterface *editable_parent_iface;
 
+void
+dzl_suggestion_entry_set_activate_on_single_click (DzlSuggestionEntry *self,
+                                                   gboolean            activate_on_single_click)
+{
+  DzlSuggestionEntryPrivate *priv = dzl_suggestion_entry_get_instance_private (self);
+
+  g_assert (DZL_IS_SUGGESTION_ENTRY (self));
+
+  activate_on_single_click = !!activate_on_single_click;
+
+  if (activate_on_single_click != priv->activate_on_single_click)
+    {
+      priv->activate_on_single_click = activate_on_single_click;
+      _dzl_suggestion_popover_set_click_mode (priv->popover, activate_on_single_click);
+      g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_ACTIVATE_ON_SINGLE_CLICK]);
+    }
+}
+
+gboolean
+dzl_suggestion_entry_get_activate_on_single_click (DzlSuggestionEntry *self)
+{
+  DzlSuggestionEntryPrivate *priv = dzl_suggestion_entry_get_instance_private (self);
+
+  g_return_val_if_fail (DZL_IS_SUGGESTION_ENTRY (self), FALSE);
+
+  return priv->activate_on_single_click;
+}
+
 static void
 dzl_suggestion_entry_show_suggestions (DzlSuggestionEntry *self)
 {
@@ -90,7 +124,7 @@ dzl_suggestion_entry_show_suggestions (DzlSuggestionEntry *self)
   DZL_EXIT;
 }
 
-static void
+void
 dzl_suggestion_entry_hide_suggestions (DzlSuggestionEntry *self)
 {
   DzlSuggestionEntryPrivate *priv = dzl_suggestion_entry_get_instance_private (self);
@@ -99,7 +133,8 @@ dzl_suggestion_entry_hide_suggestions (DzlSuggestionEntry *self)
 
   g_assert (DZL_IS_SUGGESTION_ENTRY (self));
 
-  dzl_suggestion_popover_popdown (priv->popover);
+  if (priv->popover != NULL)
+    dzl_suggestion_popover_popdown (priv->popover);
 
   DZL_EXIT;
 }
@@ -151,7 +186,7 @@ dzl_suggestion_entry_key_press_event (GtkWidget   *widget,
    * If Tab was pressed, and there is uncommitted suggested text,
    * commit it and stop propagation of the key press.
    */
-  if (key->keyval == GDK_KEY_Tab && (key->state & GDK_MODIFIER_MASK) == 0)
+  if (key->keyval == GDK_KEY_Tab && (key->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) == 0)
     {
       const gchar *typed_text;
       DzlSuggestion *suggestion;
@@ -225,19 +260,6 @@ dzl_suggestion_entry_changed (GtkEditable *editable)
 
   g_assert (DZL_IS_SUGGESTION_ENTRY (self));
 
-  /*
-   * If we aren't focused, just ignore everything. Additionally, if the text is
-   * being changed outside of a key-press-event, we want to ignore completion
-   * there too (such as calling set_text() while focused).
-   *
-   * One such example might be updating an URI in a webbrowser.
-   *
-   * We should be okay on the a11y/IM front here too, since that should happen
-   * via synthesized events.
-   */
-  if (!priv->in_key_press || !gtk_widget_has_focus (GTK_WIDGET (editable)))
-    goto update;
-
   g_signal_handler_block (self, priv->changed_handler);
 
   text = dzl_suggestion_entry_buffer_get_typed_text (priv->buffer);
@@ -245,7 +267,7 @@ dzl_suggestion_entry_changed (GtkEditable *editable)
   if (text == NULL || *text == '\0')
     {
       dzl_suggestion_entry_buffer_set_suggestion (priv->buffer, NULL);
-      g_signal_emit (self, signals [HIDE_SUGGESTIONS], 0);
+      dzl_suggestion_entry_hide_suggestions (self);
       DZL_GOTO (finish);
     }
 
@@ -265,12 +287,29 @@ finish:
 
   g_signal_handler_unblock (self, priv->changed_handler);
 
-update:
   dzl_suggestion_entry_update_attrs (self);
 
   DZL_EXIT;
 }
 
+static void
+dzl_suggestion_entry_real_suggestion_activated (DzlSuggestionEntry *self,
+                                                DzlSuggestion      *suggestion)
+{
+  DzlSuggestionEntryPrivate *priv = dzl_suggestion_entry_get_instance_private (self);
+
+  DZL_ENTRY;
+
+  g_assert (DZL_IS_SUGGESTION_ENTRY (self));
+  g_assert (DZL_IS_SUGGESTION (suggestion));
+
+  gtk_entry_set_text (GTK_ENTRY (self), "");
+  dzl_suggestion_entry_buffer_clear (priv->buffer);
+  g_signal_emit (self, signals [HIDE_SUGGESTIONS], 0);
+
+  DZL_EXIT;
+}
+
 static void
 dzl_suggestion_entry_suggestion_activated (DzlSuggestionEntry   *self,
                                            DzlSuggestion        *suggestion,
@@ -283,8 +322,6 @@ dzl_suggestion_entry_suggestion_activated (DzlSuggestionEntry   *self,
   g_assert (DZL_IS_SUGGESTION_POPOVER (popover));
 
   g_signal_emit (self, signals [SUGGESTION_ACTIVATED], 0, suggestion);
-  g_signal_emit (self, signals [HIDE_SUGGESTIONS], 0);
-  gtk_entry_set_text (GTK_ENTRY (self), "");
 
   DZL_EXIT;
 }
@@ -297,7 +334,11 @@ dzl_suggestion_entry_move_suggestion (DzlSuggestionEntry *self,
 
   g_assert (DZL_IS_SUGGESTION_ENTRY (self));
 
+  priv->in_move_by++;
+
   dzl_suggestion_popover_move_by (priv->popover, amount);
+
+  priv->in_move_by--;
 }
 
 static void
@@ -314,6 +355,27 @@ dzl_suggestion_entry_activate_suggestion (DzlSuggestionEntry *self)
   DZL_EXIT;
 }
 
+static void
+dzl_suggestion_entry_notify_selected_cb (DzlSuggestionEntry   *self,
+                                         GParamSpec           *pspec,
+                                         DzlSuggestionPopover *popover)
+{
+  DzlSuggestionEntryPrivate *priv = dzl_suggestion_entry_get_instance_private (self);
+
+  g_assert (DZL_IS_SUGGESTION_ENTRY (self));
+  g_assert (DZL_IS_SUGGESTION_POPOVER (popover));
+
+  if (priv->in_move_by > 0)
+    {
+      DzlSuggestion *suggestion = dzl_suggestion_popover_get_selected (priv->popover);
+
+      if (suggestion != NULL)
+        g_signal_emit (self, signals [SUGGESTION_SELECTED], 0, suggestion);
+    }
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SUGGESTION]);
+}
+
 static void
 dzl_suggestion_entry_constructed (GObject *object)
 {
@@ -359,10 +421,18 @@ dzl_suggestion_entry_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_ACTIVATE_ON_SINGLE_CLICK:
+      g_value_set_boolean (value, dzl_suggestion_entry_get_activate_on_single_click (self));
+      break;
+
     case PROP_MODEL:
       g_value_set_object (value, dzl_suggestion_entry_get_model (self));
       break;
 
+    case PROP_SUGGESTION:
+      g_value_set_object (value, dzl_suggestion_entry_get_suggestion (self));
+      break;
+
     case PROP_TYPED_TEXT:
       g_value_set_string (value, dzl_suggestion_entry_get_typed_text (self));
       break;
@@ -382,10 +452,18 @@ dzl_suggestion_entry_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_ACTIVATE_ON_SINGLE_CLICK:
+      dzl_suggestion_entry_set_activate_on_single_click (self, g_value_get_boolean (value));
+      break;
+
     case PROP_MODEL:
       dzl_suggestion_entry_set_model (self, g_value_get_object (value));
       break;
 
+    case PROP_SUGGESTION:
+      dzl_suggestion_entry_set_suggestion (self, g_value_get_object (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -410,6 +488,23 @@ dzl_suggestion_entry_class_init (DzlSuggestionEntryClass *klass)
   klass->hide_suggestions = dzl_suggestion_entry_hide_suggestions;
   klass->show_suggestions = dzl_suggestion_entry_show_suggestions;
   klass->move_suggestion = dzl_suggestion_entry_move_suggestion;
+  klass->suggestion_activated = dzl_suggestion_entry_real_suggestion_activated;
+
+  /**
+   * DzlSuggestionEntry:activate-on-single-click:
+   *
+   * The "activate-on-single-click" property denotes if results should be
+   * activated simply by clicking on them. You may want to set this to
+   * %FALSE if you want the behavior to only select the item.
+   *
+   * Since: 3.30
+   */
+  properties [PROP_ACTIVATE_ON_SINGLE_CLICK] =
+    g_param_spec_boolean ("activate-on-single-click",
+                          "Activate on Single Click",
+                          "If entries should be activated upon a single click",
+                          FALSE,
+                          (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
   properties [PROP_MODEL] =
     g_param_spec_object ("model",
@@ -425,6 +520,20 @@ dzl_suggestion_entry_class_init (DzlSuggestionEntryClass *klass)
                          NULL,
                          (G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * DzlSuggestionEntry:suggestion:
+   *
+   * The "suggestion" property is the currently selected suggestion, if any.
+   *
+   * Since: 3.30
+   */
+  properties [PROP_SUGGESTION] =
+    g_param_spec_object ("suggestion",
+                         "Suggestion",
+                         "The currently selected suggestion",
+                         DZL_TYPE_SUGGESTION,
+                         (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_properties (object_class, N_PROPS, properties);
 
   signals [HIDE_SUGGESTIONS] =
@@ -462,6 +571,32 @@ dzl_suggestion_entry_class_init (DzlSuggestionEntryClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (DzlSuggestionEntryClass, suggestion_activated),
                   NULL, NULL, NULL, G_TYPE_NONE, 1, DZL_TYPE_SUGGESTION);
+  g_signal_set_va_marshaller (signals [SUGGESTION_ACTIVATED],
+                              G_TYPE_FROM_CLASS (klass),
+                              g_cclosure_marshal_VOID__OBJECTv);
+
+  /**
+   * DzlSuggestionEntry::suggestion-selected:
+   * @self: a #DzlSuggestionEntry
+   * @suggestion: a #DzlSuggestion
+   *
+   * This signal is emitted when a selection has been specifically selected
+   * by the user, such as by clicking on the row or moving to the row with
+   * keyboard, such as with #DzlSuggestionEntry::move-suggestion
+   *
+   * Since: 3.30
+   */
+  signals [SUGGESTION_SELECTED] =
+    g_signal_new ("suggestion-selected",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (DzlSuggestionEntryClass, suggestion_selected),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1, DZL_TYPE_SUGGESTION);
+  g_signal_set_va_marshaller (signals [SUGGESTION_SELECTED],
+                              G_TYPE_FROM_CLASS (klass),
+                              g_cclosure_marshal_VOID__OBJECTv);
 
   widget_class->activate_signal = signals [ACTIVATE_SUGGESTION] =
     g_signal_new_class_handler ("activate-suggestion",
@@ -491,6 +626,7 @@ dzl_suggestion_entry_init (DzlSuggestionEntry *self)
 {
   DzlSuggestionEntryPrivate *priv = dzl_suggestion_entry_get_instance_private (self);
 
+  priv->activate_on_single_click = TRUE;
   priv->func = dzl_suggestion_entry_default_position_func;
 
   priv->changed_handler =
@@ -505,6 +641,11 @@ dzl_suggestion_entry_init (DzlSuggestionEntry *self)
                                 "relative-to", self,
                                 "type", GTK_WINDOW_POPUP,
                                 NULL);
+  g_signal_connect_object (priv->popover,
+                           "notify::selected",
+                           G_CALLBACK (dzl_suggestion_entry_notify_selected_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->popover,
                            "suggestion-activated",
                            G_CALLBACK (dzl_suggestion_entry_suggestion_activated),
